use reqwest::StatusCode;

pub struct Request<'a> {
    pub base_url: &'a str,
    pub full_url: Option<&'a str>,
}

impl<'a> Request<'a> {
    pub fn new() -> Self {
        Request {
            base_url: "https://site.api.espn.com/apis/site/v2/sports/football",
            full_url: None,
        }
    }

    pub fn base_url(&self) -> &str {
        self.base_url
    }

    pub fn set_base_url(&mut self, base_url: &'a str) {
        self.base_url = base_url;
    }

    pub async fn request_url<T>(&self, url: &String) -> Result<T, Box<dyn std::error::Error>>
    where
        T: for<'de> serde::Deserialize<'de>,
    {
        println!("{}", url);
        let api_result = match reqwest::get(url).await {
            Ok(r) => r,
            Err(e) => return Err(Box::new(e)),
        };
        match api_result.status() {
            StatusCode::OK => {
                let json = api_result.json::<T>();
                let json_result = match json.await {
                    Ok(j) => j,
                    Err(e) => return Err(Box::new(e)),
                };
                Ok(json_result)
            }
            status => Err(Box::new(std::io::Error::new(
                std::io::ErrorKind::Other,
                format!("Unexpected status code: {}", status),
            ))),
        }
    }
}

pub struct RequestOptions {
    team_id: Option<u32>,
    event_id: Option<u32>,
    year: Option<u32>,
}
